<?php
/**
 * @version    1.0.2
 * @package    com_gadecisions
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gadecisions\Site\Model;

// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\Utilities\ArrayHelper;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\MVC\Model\FormModel;
use \GlennArkell\Component\Gadecisions\Administrator\Helper\GadecisionsHelper;

/**
 * Form model.
 * @since  1.6
 */
class MotionformModel extends FormModel
{
    private $item = null;

    /**
     * Method to auto-populate the model state.
     * Note. Calling getState in this method will result in recursion.
     * @return void
     * @since  1.6
     * @throws Exception
     */
    protected function populateState()
    {
        $app = Factory::getApplication('com_gadecisions');

        // Load state from the request userState on edit or from the passed variable on default
        if ($app->input->get('layout') == 'edit') {
                $id = $app->getUserState('com_gadecisions.edit.motion.id');
        } else {
                $id = $app->input->get('id');
                $app->setUserState('com_gadecisions.edit.motion.id', $id);
        }

        $this->setState('motion.id', $id);

        // Load the parameters.
        $params       = $app->getParams();
        $params_array = $params->toArray();

        if (isset($params_array['menu_motion_id']) && $params_array['menu_motion_id']) {
            $this->setState('motion.id', $params_array['menu_motion_id']);
        }

        $this->setState('params', $params);
    }

    /**
     * Method to get an ojbect.
     * @param   integer $id The id of the object to get.
     * @return Object|boolean Object on success, false on failure.
     * @throws Exception
     */
    public function getItem($id = null)
    {
        if ($this->item === null) {
            $this->item = false;

            if (empty($id)) {
                    $id = $this->getState('motion.id');
            }

            // Get a level row instance.
            $table = $this->getTable();

            if ($table !== false && $table->load($id) && !empty($table->id)) {
                $user = GadecisionsHelper::getSpecificUser();
                $id   = $table->id;

                $canAdmin = $user->authorise('core.admin', 'com_gadecisions');
                $canVote = $user->authorise('core.vote', 'com_gadecisions');

                if ($table->group_id) {
                    $canVote = $canVote && !in_array($table->group_id, $user->groups) ? false : $canVote;
                }
                
                $authorised = $canVote || $canAdmin ? true : false;

                if (!$authorised) {
                    throw new \Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
                }

                // Check published state.
                if ($published = $this->getState('filter.published')) {
                    if (isset($table->state) && $table->state != $published) {
                        return $this->item;
                    }
                }

                // Convert the Table to a clean Object.
                $properties = $table->getProperties(1);
                $this->item = ArrayHelper::toObject($properties, 'stdClass');

            }
        }

        return $this->item;
    }

    /**
     * Method to get the table
     * @param   string $type   Name of the Table class
     * @param   string $prefix Optional prefix for the table class name
     * @param   array  $config Optional configuration array for Table object
     * @return  Table|boolean Table if found, boolean false on failure
     */
    public function getTable($type = 'Motion', $prefix = 'Administrator', $config = array())
    {
        return parent::getTable($type, $prefix, $config);
    }

    /**
     * Get an item by alias
     * @param   string $alias Alias string
     * @return int Element id
     */
    public function getItemIdByAlias($alias)
    {
        $table      = $this->getTable();
        $properties = $table->getProperties();

        if (!in_array('alias', $properties)) {
                return null;
        }

        $table->load(array('alias' => $alias));
        $id = $table->id;

        return $id;
        
    }

    /**
     * Method to check in an item.
     * @param   integer $id The id of the row to check out.
     * @return  boolean True on success, false on failure.
     * @since    1.6
     */
    public function checkin($id = null)
    {
        // Get the id.
        $id = (!empty($id)) ? $id : (int) $this->getState('motion.id');
        
        if ($id) {
            // Initialise the table
            $table = $this->getTable();

            // Attempt to check the row in.
            if (method_exists($table, 'checkin')) {
                if (!$table->checkin($id)) {
                    return false;
                }
            }
        }

        return true;
        
    }

    /**
     * Method to check out an item for editing.
     * @param   integer $id The id of the row to check out.
     * @return  boolean True on success, false on failure.
     * @since    1.6
     */
    public function checkout($id = null)
    {
        // Get the user id.
        $id = (!empty($id)) ? $id : (int) $this->getState('motion.id');
        
        if ($id) {
            // Initialise the table
            $table = $this->getTable();

            // Get the current user object.
            $user = GadecisionsHelper::getSpecificUser();

            // Attempt to check the row out.
            if (method_exists($table, 'checkout')) {
                if (!$table->checkout($user->id, $id)) {
                    return false;
                }
            }
        }

        return true;
        
    }

    /**
     * Method to get the form.
     * The base form is loaded from XML
     * @param   array   $data     An optional array of data for the form to interogate.
     * @param   boolean $loadData True if the form is to load its own data (default case), false if not.
     * @return  Form    A Form object on success, false on failure
     * @since   1.6
     */
    public function getForm($data = array(), $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm('com_gadecisions.motion', 'motionform', array(
                    'control'   => 'jform',
                    'load_data' => $loadData ) );

        if (empty($form)) {
                return false;
        }
        
        //FieldsPlugin::onContentPrepareForm($form, $data);

        return $form;
    }

    /**
     * Method to get the data that should be injected in the form.
     * @return    array  The default data is an empty array.
     * @since    1.6
     */
    protected function loadFormData()
    {
        $data = Factory::getApplication()->getUserState('com_gadecisions.edit.motion.data', array());

        if (empty($data)) {
            $data = $this->getItem();
        }

        if ($data) {
            return $data;
        }

        return array();
    }

    /**
     * Method to save the form data.
     * @param   array $data The form data
     * @return bool
     * @throws Exception
     * @since 1.6
     */
    public function save($data)
    {
        $id    = (!empty($data['id'])) ? $data['id'] : (int) $this->getState('motion.id');
        $state = (!empty($data['state'])) ? 1 : 0;
        $user  = GadecisionsHelper::getSpecificUser();

        if ($id) {
            // Check the user can edit this item
            $authorised = $user->authorise('core.edit', 'com_gadecisions') || $authorised = $user->authorise('core.edit.own', 'com_gadecisions');
        } else {
            // Check the user can create new items in this section
            $authorised = $user->authorise('core.create', 'com_gadecisions');
        }

        if ($authorised !== true) {
            throw new \Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

         $table = $this->getTable();

        if ($table->save($data) === true) {
            return $table->id;
        } else {
            return false;
        }
        
    }

    /**
     * Method to delete data
     * @param   int $pk Item primary key
     * @return  int  The id of the deleted item
     * @throws Exception
     * @since 1.6
     */
    public function delete($pk)
    {
        $user = GadecisionsHelper::getSpecificUser();

        if (empty($pk)) {
            $pk = (int) $this->getState('motion.id');
        }

        if ($pk == 0 || $this->getItem($pk) == null) {
            throw new \Exception(Text::_('COM_GADECISIONS_ITEM_DOESNT_EXIST'), 404);
        }

        if ($user->authorise('core.delete', 'com_gadecisions') !== true) {
            throw new \Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();

        if ($table->delete($pk) !== true) {
            throw new \Exception(Text::_('JERROR_FAILED'), 501);
        }

        return $pk;
        
    }

    /**
     * Check if data can be saved
     * @return bool
     */
    public function getCanSave()
    {
        $table = $this->getTable();

        return $table !== false;
    }
    
    /**
     * Method to save the vote against the user.
     * @param   array $data The form data
     * @return bool
     * @throws Exception
     * @since 1.6
     */
    public function castVote($data)
    {
        $id    = (!empty($data['id'])) ? $data['id'] : (int) $this->getState('motion.id');
        $today = Factory::getDate()->toSql();
        $user  = GadecisionsHelper::getSpecificUser();

        $table = $this->getTable();

		$table->load($id);

        // test if motion passed date and is decider vote
        $decider = $table->expiry_date < $today && $table->decider == $user->id ? $user->id : 0;


        $new_rec = new \stdClass();
        $new_rec->id = 0;
        $new_rec->state = 1;
        $new_rec->created_date = $today;
        $new_rec->created_by = $user->id;
        $new_rec->motion_id = $id;
        $new_rec->user_id = $data['user_id'];
        $new_rec->decider = $decider;
        $new_rec->comment = '';

		try {
			$result = Factory::getContainer()->get('DatabaseDriver')->insertObject('#__gadecisions_voters', $new_rec);
		} catch (RuntimeException $e) {
			return false;
		}
		
        if ($data['field_id'] && !isset($data['vote'])) {
            // get fields and data
            foreach ($data['custfldmotions'] AS $fname => $fvalue) {
                if (is_array($fvalue)) {
                    // not catering for multiple selects including checkbox fields
                } else {
                    unset($new_rec->user_id);
                    unset($new_rec->decider);
                    $new_rec->id = 0;
                    $new_rec->created_by = 0;
                    $new_rec->field_id = $data['field_id'];
                    $new_rec->vote = $fvalue;
                    $new_rec->comment = $fname;
                    $result = Factory::getContainer()->get('DatabaseDriver')->insertObject('#__gadecisions_cfvotes', $new_rec);
                }
            }
            return true;
        } else {
    		if ($data['vote']) {
                $table->votes_for = $table->votes_for + 1;
            } else {
                $table->votes_against = $table->votes_against + 1;
            }

    		return $table->store(true);
		}

    }

    /**
     * Method to save the vote against the user.
     * @param   array $data The form data
     * @return bool
     * @throws Exception
     */
    public function setNominee($data)
    {
        $id    = (!empty($data['id'])) ? $data['id'] : (int) $this->getState('motion.id');

        $table = $this->getTable();

		$table->load($id);
		$table->decider = $data['decider'];

		return $table->store(true);

    }

}
